/*
 * Copyright 2017-2018 by Kappich Systemberatung, Aachen
 *
 * This file is part of de.kappich.pat.gnd.
 *
 * de.kappich.pat.gnd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.kappich.pat.gnd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.kappich.pat.gnd.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436
 * mail: <info@kappich.de>
 */

package de.kappich.pat.gnd.kmPlugin;

import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.kappich.annotations.Nullable;
import de.kappich.pat.gnd.displayObjectToolkit.DOTDefinitionDialog;
import de.kappich.pat.gnd.gnd.PluginManager;
import de.kappich.pat.gnd.pluginInterfaces.*;
import de.kappich.pat.gnd.properties.*;

/**
 * {@code DOTKmPlugin} ist die Implementation von {@link DisplayObjectTypePlugin} im Km-Plugin.
 *
 * @author Kappich Systemberatung
 * @version $Revision$
 */
public final class DOTKmPlugin implements DisplayObjectTypePlugin {

	private static final Debug _debug = Debug.getLogger();

	/**
	 * Der Konstruktor.
	 */
	public DOTKmPlugin() {
		super();
		// prüft, ob die Implementation im PluginManager bekannt ist
		if (!PluginManager.hasPlugin(getName())) {
			_debug.error("Das DOTKmPlugin ist dem PluginManager nicht bekannt.");
		}
	}

	@Override
	public String getName() {
		return "Betriebskilometrierung";
	}

	@Override
	public DisplayObjectType getDisplayObjectType() {
		return new DOTKm();
	}

	@Override
	public DOTDefinitionPanel getPanel(final DOTDefinitionDialog dotDefinitionDialog) {
		return new DefaultDOTDefinitionPanel(new DOTKmPlugin(), dotDefinitionDialog);
	}

	@Override
	public DisplayObjectsInitializer getInitializer() { return new KmInitializer(); }

	@Override
	public DisplayObjectPainter getPainter() {
		return new DOTKmPainter();
	}

	@Override
	public String[] getPrimitiveFormTypes() {
		return new String[0];
	}

	@Override
	@Nullable
	public Property[] getProperties(final Object o) {
		Property[] properties = null;
		if(o == null) {
			properties = new Property[]{
					ColorProperty.getInstance(), TextSizeProperty.getInstance(), KmFormatProperty.getInstance(),
					DottingProperty.getInstance(), TextStyleProperty.getInstance(), DistanceRasterProperty.getInstance()
			};
		}
		return properties;
	}

	@Override
	public String getGeometryType() {
		return "typ.straßenTeilSegment";
	}

	@Override
	public boolean isSystemObjectTypeSupported(final DataModel configuration, final SystemObjectType systemObjectType) {
		final SystemObjectType geoReferenceObjectType = configuration.getType(getGeometryType());
		return systemObjectType.equals(geoReferenceObjectType);
	}

	@Override
	public boolean isDynamicsPossible() {
		return false;
	}

	@Override
	public boolean isMapScaleListeningNecessary() {
		return false;
	}
}
